--[[
name, rank, icon, count,  dispelType, duration, expires, caster,  isStealable, shouldConsolidate, spellID  = UnitAura("unit", index [,  "filter"]) or UnitAura("unit", i)
]]

local lib = LibStub("LibAuraInfo-1.0");
if(not lib) then error("API.lua for LibAuraInfo needs library loaded."); return; end

local debugPrint = lib.debugPrint
local strsplit = strsplit
local tonumber = tonumber
local UnitName = UnitName
local unpack = unpack
local GetTime = GetTime

local function GUIDIsPlayer(guid)
	local B = tonumber(guid:sub(5,5), 16);
	local maskedB = B % 8; -- x % 8 has the same effect as x & 0x7 on numbers <= 0xf
--~ 	local knownTypes = {[0]="player", [3]="NPC", [4]="pet", [5]="vehicle"};
--~ 	print("Your target is a " .. (knownTypes[maskedB] or " unknown entity!"));
	if maskedB == 0 then
		return true
	end
	return false
end

--Return the duration of a spell.
function lib:GetDuration(spellID, srcGUID, dstGUID, dstIsPlayer)
	local dstIsPlayer = dstIsPlayer or dstGUID and GUIDIsPlayer(dstGUID) or false
	local spellStr = ""
	if dstIsPlayer and lib.auraInfoPvP[spellID] then
		--Receiver is a player and the spell has a PvP duration. Return the pvp duration.
		local duration = lib.auraInfoPvP[spellID]
		
		if dstGUID and duration then
			--Check if there's dimminshing returns on the spell.
			duration = self:GetDRDuration(dstGUID, spellID, duration)
		end
				
		return tonumber(duration or 0)
	elseif self.auraInfo[spellID] then
		--Check caster GUID was given.
		if srcGUID then
			--Check if we've seen that caster cast a spell with a duration that doesn't match our own (spec/glphed into something?)
			if self.GUIDDurations[srcGUID.."-"..spellID] then
				local dur = self.GUIDDurations[srcGUID.."-"..spellID]
				
				--Check if receiver GUID was given.
				if dstGUID then
					--Check if there's dimminshing returns on the spell.
					dur = self:GetDRDuration(dstGUID, spellID, dur)
				end
				return dur
			end
		end
	
		local duration, debuffType = strsplit(";", self.auraInfo[spellID])
		return tonumber(duration or 0)
	end
end

--Return the debuff type of a spell.
function lib:GetDebuffType(spellID)
	if self.auraInfo[spellID] then
		local duration, debuffType = strsplit(";", self.auraInfo[spellID])
		if debuffType then
			debuffType = tonumber(debuffType)
			if debuffType == 0 then
				return "none" --Lowercase because DebuffTypeColor["none"] is lowercase.
			else
				return self.debuffTypes[debuffType] or "unknown"
			end
		end
		return "none"--Lowercase because DebuffTypeColor["none"] is lowercase.
	end
	return nil
end

function lib:GetNumGUIDAuras(dstGUID)
	self:RemoveExpiredAuras(dstGUID)
	
	if self.GUIDAuras[dstGUID] then
		return #self.GUIDAuras[dstGUID]
	end

	return 0
end

function lib:GUIDAura(dstGUID, i)
	if self.GUIDAuras[dstGUID] and self.GUIDAuras[dstGUID][i] then
		local data = self.GUIDAuras[dstGUID][i]
		return true, data.count or 0, data.debuffType, data.duration, data.expirationTime, data.isDebuff, data.srcGUID
	end
	return false
end


function lib:GUIDAuraID(dstGUID, spellID, srcGUID)
	if self.GUIDAuras[dstGUID] then
		local data
		if srcGUID then
			for i=1, #self.GUIDAuras[dstGUID] do 
				data = self.GUIDAuras[dstGUID][i]
				if data.spellID == spellID and data.srcGUID == srcGUID then
					return true, data.count or 0, data.debuffType, data.duration, data.expirationTime, data.isDebuff, data.srcGUID
				end
			end
		end
		for i=1, #self.GUIDAuras[dstGUID] do 
			data = self.GUIDAuras[dstGUID][i]
			if data.spellID == spellID then
				return true, data.count or 0, data.debuffType, data.duration, data.expirationTime, data.isDebuff, data.srcGUID
			end
		end
	end
	return false
end
--[[
old function no longer used.
function lib:GetGUIDAuras(GUID)
	if self.GUIDAuras[GUID] then
		return unpack(self.GUIDAuras[GUID])
	end
	return nil
end
]]

function lib:HasDREffect(dstGUID, spellID)
	local drType = self.drSpells[spellID]
	if drType then
		local key = dstGUID..drType
		local reset = self.GUIDDrEffects_reset[key]
		if reset and GetTime() < reset then
			return true, (self.GUIDDrEffects_diminished[key] or 1)
		end
	end
	return false
end

function lib:GetGUIDInfo(GUID)
	return self.GUIDData_name[GUID], self.GUIDData_flags[GUID]
end
