local lib = LibStub("LibAuraInfo-1.0");
if(not lib) then error("combatlog.lua for LibAuraInfo needs library loaded."); return; end

local debugPrint = lib.debugPrint
local GetTime = GetTime
local pairs = pairs
local table_remove = table.remove
local table_insert = table.insert
local bit_band = bit.band
local COMBATLOG_OBJECT_TYPE_PLAYER = COMBATLOG_OBJECT_TYPE_PLAYER
local GetSpellInfo = GetSpellInfo
local tostring = tostring

lib.GUIDAuras = lib.GUIDAuras or {}

lib.GUIDData_name = {}
lib.GUIDData_flags = {}
local function SaveGUIDInfo(guid, name, flags)
	lib.GUIDData_name[guid] = name
	lib.GUIDData_flags[guid] = flags
end


function lib.frame:COMBAT_LOG_EVENT_UNFILTERED(event, ...)
	--timestamp, eventType, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags
	local timestamp, eventType, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags  = ...-- ***

	if srcGUID and not lib.GUIDData_name[srcGUID] then
		SaveGUIDInfo(srcGUID, srcName, srcFlags)
	end
	if dstGUID and not lib.GUIDData_name[dstGUID] then
		SaveGUIDInfo(dstGUID, dstName, dstFlags)
	end
	
	if self[eventType] then
		self[eventType](self, eventType, ...)
	end
end

----------------------------------------------
function lib:RemoveExpiredAuras(dstGUID)	--
-- Remove expired auras from a GUID. 		--
----------------------------------------------
	if self.GUIDAuras[dstGUID] then
		local data
		local getTime = GetTime()
		for i = #self.GUIDAuras[dstGUID], 1, -1 do 
			data = self.GUIDAuras[dstGUID][i]
			if getTime > data.expirationTime then
				table_remove(self.GUIDAuras[dstGUID], i)
			end
		end
	end
end

----------------------------------------------------------------------
function lib:checkIfAuraAlreadyOnGUID(dstGUID, spellID, srcGUID)	--
-- Used for debuging and learning the combatlog. 					--
----------------------------------------------------------------------

	local data
	local spellName = GetSpellInfo(spellID)
	for i= #self.GUIDAuras[dstGUID], 1, -1  do 
		data = self.GUIDAuras[dstGUID][i]
		if data.spellID == spellID  then --and srcGUID ~= data.srcGUID
			if srcGUID ~= data.srcGUID then
				local newName = self:GetGUIDInfo(srcGUID)
				local oldName = self:GetGUIDInfo(data.srcGUID)
				if newName == oldName then
					debugPrint("checkIfAuraAlreadyOnGUID", spellID, spellName, "new:"..tostring(srcGUID), "old:"..tostring(data.srcGUID))
				else
					debugPrint("checkIfAuraAlreadyOnGUID", spellID, spellName, "new:"..tostring(newName), "old:"..tostring(oldName))
				end
			end
		end
	
	end
end

------------------------------------------------------------------
function lib:AddAuraToGUID(dstGUID, spellID, srcGUID, isDebuff)	--
-- Add a auraID to our GUID list.								--
------------------------------------------------------------------

	local duration = self:GetDuration(spellID, srcGUID, dstGUID)
	local debuffType = self:GetDebuffType(spellID)
	local currentTime = GetTime()

	self.GUIDAuras[dstGUID] = self.GUIDAuras[dstGUID] or {}

	--[===[@debug@
	self:checkIfAuraAlreadyOnGUID(dstGUID, spellID, srcGUID)
	--@end-debug@]===]
	
	--[[
		I didn't want to use tables this way when I started the project but due to multiple instnces of a spellID being on a GUID I couldn't do a hash table lookup.
		I wanted do something like
		GUIDAuras_Duration[dstGUID..spellID..srcGUID] = 30
		but because UnitAura() sometimes doesn't return a unitID to get srcGUID, I had to do a index table. meh
		
	]]

	table_insert(self.GUIDAuras[dstGUID], #self.GUIDAuras[dstGUID]+1, {
		spellID = spellID,
		srcGUID = srcGUID,
		duration = duration,
		debuffType = debuffType,
		isDebuff = isDebuff,
		expirationTime = currentTime + duration,
		startTime = currentTime,
	})
	
	
--~ 	table.sort(self.GUIDAuras[dstGUID], function(a,b) 
--~ 		return a.expirationTime < b.expirationTime 
--~ 	end)
end

------------------------------------------------------
function lib:AddAuraDose(dstGUID, spellID, srcGUID)	--
-- Increase stack count of a aura.					--
------------------------------------------------------

	if self.GUIDAuras[dstGUID] then
		local data
		if srcGUID then
			for i=1, #self.GUIDAuras[dstGUID] do 
				data = self.GUIDAuras[dstGUID][i]
				if data.spellID == spellID and data.srcGUID == srcGUID then
					
					data.stackCount = (data.stackCount or 1) + 1
					
					data.startTime = GetTime()
					data.expirationTime = data.duration + data.startTime
					
					return true, data.stackCount, data.expirationTime
				end
			end
		end
		
		for i=1, #self.GUIDAuras[dstGUID] do 
			data = self.GUIDAuras[dstGUID][i]
			if data.spellID == spellID then
				
				data.stackCount = (data.stackCount or 1) + 1
				
				data.startTime = GetTime()
				data.expirationTime = data.duration + data.startTime
				
				return true, data.stackCount, data.expirationTime
			end
		end

	end
	return false
end

----------------------------------------------------------
function lib:RemoveAuraDose(dstGUID, spellID, srcGUID)	--
-- Remove 1 stack from a aura.							--
----------------------------------------------------------
	if self.GUIDAuras[dstGUID] then
		local data
		if srcGUID then
			for i=1, #self.GUIDAuras[dstGUID] do 
				data = self.GUIDAuras[dstGUID][i]
				if data.spellID == spellID and data.srcGUID == srcGUID then
					data.stackCount = (data.stackCount or 1) - 1
--~ 					data.startTime = GetTime()
--~ 					data.expirationTime = data.duration + data.startTime
					return true, data.stackCount, data.expirationTime
				end
			end
		end
		
		for i=1, #self.GUIDAuras[dstGUID] do 
			data = self.GUIDAuras[dstGUID][i]
			if data.spellID == spellID then
				data.stackCount = (data.stackCount or 1) - 1
--~ 				data.startTime = GetTime()
--~ 				data.expirationTime = data.duration + data.startTime
				return true, data.stackCount, data.expirationTime
			end
		end

	end
	return false
end

------------------------------------------------------
function lib:RefreshAura(dstGUID, spellID, srcGUID)	--
-- Refresh the start and expiration time of a aura.	--
------------------------------------------------------
	if self.GUIDAuras[dstGUID] then
		local data
		if srcGUID then
			for i=1, #self.GUIDAuras[dstGUID] do 
				data = self.GUIDAuras[dstGUID][i]
				if data.spellID == spellID and data.srcGUID == srcGUID then
					data.startTime = GetTime()
					data.expirationTime = data.duration + data.startTime
					return true, data.expirationTime
				end
			end
		end
		
		for i=1, #self.GUIDAuras[dstGUID] do 
			data = self.GUIDAuras[dstGUID][i]
			if data.spellID == spellID then
				data.startTime = GetTime()
				data.expirationTime = data.duration + data.startTime
				return true, data.expirationTime
			end
		end

	end
	return false
end

------------------------------------------------------------------
local function RemoveAuraFromGUID(dstGUID, spellID, srcGUID)	--
-- Remove a aura from a GUID.									--
------------------------------------------------------------------

	if lib.GUIDAuras[dstGUID] then
		local data
		local spellName  = GetSpellInfo(spellID)
		
		if srcGUID then
			for i= #lib.GUIDAuras[dstGUID],1, -1 do 
				data = lib.GUIDAuras[dstGUID][i]
				if data.spellID == spellID and srcGUID == data.srcGUID then
--~ 					debugPrint("RemoveAuraFromGUID", spellName, srcGUID, data.srcGUID, "match:"..tostring(srcGUID == data.srcGUID))
					table_remove(lib.GUIDAuras[dstGUID], i)
					return
				end
			end
		end
		
		for i= #lib.GUIDAuras[dstGUID],1, -1 do 
			data = lib.GUIDAuras[dstGUID][i]
			if data.spellID == spellID  then
				table_remove(lib.GUIDAuras[dstGUID], i)
				
				--[===[@debug@
				local newName = lib:GetGUIDInfo(srcGUID)
				local oldName = lib:GetGUIDInfo(data.srcGUID)
				if newName == oldName then
					debugPrint("RemoveAuraFromGUID", spellName, "new:"..tostring(newName), "old:"..tostring(data.srcGUID))
--~ 				else
--~ 					debugPrint("RemoveAuraFromGUID", spellName, "new:"..tostring(newName), "old:"..tostring(oldName))
				end
				--@end-debug@]===]
				return
			end
		end
		
	end
end

------------------------------------------------------
function lib:RemoveAllAurasFromGUID(dstGUID)		--
-- Remove all auras on a GUID. They must have died.	--
------------------------------------------------------
	if self.GUIDAuras[dstGUID] then
		for i=#self.GUIDAuras[dstGUID], 1, -1 do 
			table_remove(self.GUIDAuras[dstGUID], i)
		end
	end
end

--[[
local function AuraAlreadyOnGUID(dstGUID, spellID, srcGUID)
	if lib.GUIDAuras[dstGUID] then
		local data
		if srcGUID then
			for i=1, lib.GUIDAuras[dstGUID] do 
				data = lib.GUIDAuras[dstGUID][i]
				if data.spellID == spellID and data.srcGUID == srcGUID then
					return true
				end
			end
		end
		
		for i=1, lib.GUIDAuras[dstGUID] do 
			data = lib.GUIDAuras[dstGUID][i]
			if data.spellID == spellID then
				return true
			end
		end
	end
	return false
end
]]

function lib.frame:SPELL_AURA_REMOVED(event, timestamp, eventType, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...)	--
	local spellID, spellName, spellSchool, auraType  = ...
	RemoveAuraFromGUID(dstGUID, spellID, srcGUID)
	
	
	if lib.drSpells[spellID] then
		lib:GUIDRemovedDRAura(dstGUID, spellID, lib:FlagIsPlayer(dstFlags))
	end
end

function lib.frame:SPELL_AURA_APPLIED(event, timestamp, eventType, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...)	--
	local spellID, spellName, spellSchool, auraType, amount  = ...
	

	
	if lib.drSpells[spellID] then
		lib:GUIDGainedDRAura(dstGUID, spellID, lib:FlagIsPlayer(dstFlags))
	end
	
	if lib.auraInfo[spellID] then
		lib:RemoveExpiredAuras(dstGUID)
		
		lib:AddAuraToGUID(dstGUID, spellID, srcGUID, auraType == "DEBUFF")
		lib.callbacks:Fire("LibAuraInfo_AURA_APPLIED", dstGUID, spellID, srcGUID, spellSchool, auraType)
	end
end

function lib.frame:SPELL_AURA_REFRESH(event, timestamp, eventType, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...)
	local spellID, spellName, spellSchool, auraType  = ...
	
	if lib.drSpells[spellID] then
		lib:GUIDRemovedDRAura(dstGUID, spellID, lib:FlagIsPlayer(dstFlags))
		lib:GUIDGainedDRAura(dstGUID, spellID, lib:FlagIsPlayer(dstFlags))
	end

	local refreshed, expirationTime = lib:RefreshAura(dstGUID, spellID, srcGUID)

	if refreshed then
		lib.callbacks:Fire("LibAuraInfo_AURA_REFRESH", dstGUID, spellID, srcGUID, spellSchool, auraType, expirationTime)
		return
	end

	self:SPELL_AURA_APPLIED(event, timestamp, eventType, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...)
end

--DOSE = spell stacking
function lib.frame:SPELL_AURA_APPLIED_DOSE(event, timestamp, eventType, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...)
	local spellID, spellName, spellSchool, auraType  = ...

	local dosed, stackCount, expirationTime = lib:AddAuraDose(dstGUID, spellID, srcGUID)
	
	if dosed then
		lib.callbacks:Fire("LibAuraInfo_AURA_APPLIED_DOSE", dstGUID, spellID, srcGUID, spellSchool, auraType, stackCount, expirationTime)
		return
	end

	--Spell isn't in our list, let's add it.
	--Note this event could have fired on the 5th stack but our spell frame will only show it applied once. 
	self:SPELL_AURA_APPLIED(event, timestamp, eventType, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...)
end


function lib.frame:SPELL_AURA_APPLIED_REMOVED_DOSE(event, timestamp, eventType, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...)
	local spellID, spellName, spellSchool, auraType  = ...

	local dosed, stackCount, expirationTime = lib:RemoveAuraDose(dstGUID, spellID, srcGUID)
	
	if dosed then
		lib.callbacks:Fire("LibAuraInfo_AURA_APPLIED_DOSE", dstGUID, spellID, srcGUID, spellSchool, auraType, stackCount, expirationTime)
		return
	end
	
end


function lib.frame:SPELL_AURA_BROKEN_SPELL(...)
--~ 	local spellID, spellName, spellSchool, auraType  = ...
	self:SPELL_AURA_REMOVED(...)
end

function lib.frame:SPELL_AURA_BROKEN(...)
--~ 	local spellID, spellName, spellSchool, auraType  = ...
	self:SPELL_AURA_REMOVED(...)
end


function lib.frame:UNIT_DIED(event, timestamp, eventType, srcGUID, srcName, srcFlags, dstGUID, dstName, dstFlags, ...)
	if lib.GUIDAuras[dstGUID] then
		lib:RemoveAllAurasFromGUID(dstGUID)
		lib.callbacks:Fire("LibAuraInfo_AURA_CLEAR", dstGUID)
	end
end

function lib.frame:UNIT_DESTROYED(...)
	self:UNIT_DIED(...)
end

function lib.frame:UNIT_DISSIPATES(...)
	self:UNIT_DIED(...)
end

function lib.frame:PARTY_KILL(...)
	self:UNIT_DIED(...)
end
function lib:FlagIsPlayer(flags)
	if bit_band(flags, COMBATLOG_OBJECT_TYPE_PLAYER) == COMBATLOG_OBJECT_TYPE_PLAYER then
		return true
	end
	return nil
end
